<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model\Total\Quote;

use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Payment\Model\MethodList;
use Magento\Quote\Model\Quote\Address\Total;
use Magento\Quote\Api\Data\ShippingAssignmentInterface;
use Magento\Quote\Model\Quote;
use MSP\CashOnDelivery\Api\CashondeliveryInterface;
use Magento\Quote\Model\Quote\Address;
use Magento\Tax\Model\Config;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Tax\Model\Sales\Total\Quote\CommonTaxCollector;
use MSP\CashOnDelivery\Model\Payment;

class CashondeliveryTax extends AbstractTotal
{

    const XML_PATH_TAX_CLASS = 'tax/classes/shipping_tax_class';

    const QUOTE_TYPE = 'quote_cod';
    const CODE_QUOTE_COD = 'quote_cod';

    protected $cashOnDeliveryInterface;
    protected $priceCurrencyInterface;
    protected $taxConfig;
    protected $scopeConfigInterface;
    protected $scopeConfig;

    public function __construct(
        Payment $payment,
        PriceCurrencyInterface $priceCurrencyInterface,
        CashondeliveryInterface $cashOnDeliveryInterface,
        Config $taxConfig,
        ScopeConfigInterface $scopeConfigInterface
    ) {
        parent::__construct($payment);
        $this->cashOnDeliveryInterface = $cashOnDeliveryInterface;
        $this->priceCurrencyInterface = $priceCurrencyInterface;
        $this->taxConfig = $taxConfig;
        $this->scopeConfig = $scopeConfigInterface;
    }

    public function collect(
        Quote $quote,
        ShippingAssignmentInterface $shippingAssignment,
        Total $total
    ) {
        if ($shippingAssignment->getShipping()->getAddress()->getAddressType() != Address::TYPE_SHIPPING
            || $quote->isVirtual()
        ) {
            return $this;
        }

        $country = $quote->getShippingAddress()->getCountryModel()->getData('iso2_code');
        $region = $quote->getShippingAddress()->getRegionModel()->getData('code');

        if (!$region && is_string($quote->getShippingAddress()->getRegion())) {
            $region = $quote->getShippingAddress()->getRegion();
        }

        $baseAmount = $this->cashOnDeliveryInterface->getBaseAmount($total->getAllBaseTotalAmounts(), $country, $region);

        $baseTaxAmount = $this->cashOnDeliveryInterface->getBaseTaxAmount($baseAmount);
        $taxAmount = $this->priceCurrencyInterface->convert($baseTaxAmount);

        if ($this->_canApplyTotal($quote)) {
            $total->setBaseTotalAmount('msp_cashondelivery_tax', $baseTaxAmount);
            $total->setTotalAmount('msp_cashondelivery_tax', $taxAmount);

            $total->setBaseMspCodTaxAmount($baseTaxAmount);
            $total->setMspCodTaxAmount($taxAmount);

            $total->setBaseTaxAmount($total->getBaseTaxAmount() + $baseTaxAmount);
            $total->setTaxAmount($total->getTaxAmount() + $taxAmount);

            if ($this->taxConfig->displayCartPricesExclTax()) {
                $total->setBaseGrandTotal($total->getBaseGrandTotal() + $baseTaxAmount);
                $total->setGrandTotal($total->getGrandTotal() + $taxAmount);
            }

            $shippingTaxClassId = $this->scopeConfig->getValue(
                self::XML_PATH_TAX_CLASS,
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
                $quote->getStore()
            );

            $address = $shippingAssignment->getShipping()->getAddress();
            $associatedTaxables = $address->getAssociatedTaxables();
            if (!$associatedTaxables) {
                $associatedTaxables = [];
            }
            $amount = $this->priceCurrencyInterface->convert($baseAmount, $quote->getStore());
            $associatedTaxables[] = [
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_TYPE => self::QUOTE_TYPE,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_CODE => self::CODE_QUOTE_COD,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_UNIT_PRICE => $amount,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_BASE_UNIT_PRICE => $baseAmount,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_QUANTITY => 1,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_TAX_CLASS_ID => $shippingTaxClassId,
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_PRICE_INCLUDES_TAX => !$this->taxConfig->displayCartPricesExclTax(),
                CommonTaxCollector::KEY_ASSOCIATED_TAXABLE_ASSOCIATION_ITEM_CODE
                => CommonTaxCollector::ASSOCIATION_ITEM_CODE_FOR_QUOTE,
            ];

//            $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/erm.log');
//            $logger = new \Zend\Log\Logger();
//            $logger->addWriter($writer);
//            $logger->info($associatedTaxables);
//            $logger->info('===========================');

//            $address->setAssociatedTaxables($associatedTaxables);


        }  else {
            $baseTaxAmount = 0;
            $taxAmount = 0;
        }
        /*
         * This must be always calculated despite the method selection.
         * An empty value will result in a wrong fee preview on payment method.
         */
        $quote->setBaseMspCodTaxAmount($baseTaxAmount);
        $quote->setMspCodTaxAmount($taxAmount);

        return $this;
    }
}
